@@ -7,6 +7,7 @@ from s2clientprotocol import debug_pb2 as debug_pb
 from s2clientprotocol import query_pb2 as query_pb
 from s2clientprotocol import raw_pb2 as raw_pb
 from s2clientprotocol import sc2api_pb2 as sc_pb
+from s2clientprotocol import spatial_pb2 as spatial_pb
 
 from sc2.action import combine_actions
 from sc2.data import ActionResult, ChatChannel, Race, Result, Status
@@ -15,21 +16,23 @@ from sc2.game_info import GameInfo
 from sc2.ids.ability_id import AbilityId
 from sc2.ids.unit_typeid import UnitTypeId
 from sc2.position import Point2, Point3
-from sc2.protocol import Protocol, ProtocolError
+from sc2.protocol import ConnectionAlreadyClosed, Protocol, ProtocolError
 from sc2.renderer import Renderer
 from sc2.unit import Unit
 from sc2.units import Units
 
 
+# pylint: disable=R0904
 class Client(Protocol):
 
-    def __init__(self, ws):
+    def __init__(self, ws, save_replay_path: str = None):
         """
         :param ws:
         """
         super().__init__(ws)
         # How many frames will be waited between iterations before the next one is called
         self.game_step: int = 8
+        self.save_replay_path: Optional[str] = save_replay_path
         self._player_id = None
         self._game_result = None
         # Store a hash value of all the debug requests to prevent sending the same ones again if they haven't changed last frame
@@ -44,7 +47,7 @@ class Client(Protocol):
         self.raw_affects_selection = False
 
     @property
-    def in_game(self):
+    def in_game(self) -> bool:
         return self._status in {Status.in_game, Status.in_replay}
 
     async def join_game(self, name=None, race=None, observed_player_id=None, portconfig=None, rgb_render_config=None):
@@ -99,7 +102,7 @@ class Client(Protocol):
         return result.join_game.player_id
 
     async def leave(self):
-        """ You can use 'await self._client.leave()' to surrender midst game. """
+        """ You can use 'await self.client.leave()' to surrender midst game. """
         is_resign = self._game_result is None
 
         if is_resign:
@@ -108,19 +111,22 @@ class Client(Protocol):
             self._game_result = {self._player_id: Result.Defeat}
 
         try:
+            if self.save_replay_path is not None:
+                await self.save_replay(self.save_replay_path)
+                self.save_replay_path = None
             await self._execute(leave_game=sc_pb.RequestLeaveGame())
-        except ProtocolError:
+        except (ProtocolError, ConnectionAlreadyClosed):
             if is_resign:
                 raise
 
     async def save_replay(self, path):
-        logger.debug(f"Requesting replay from server")
+        logger.debug("Requesting replay from server")
         result = await self._execute(save_replay=sc_pb.RequestSaveReplay())
         with open(path, "wb") as f:
             f.write(result.save_replay.data)
         logger.info(f"Saved replay to {path}")
 
-    async def observation(self, game_loop=None):
+    async def observation(self, game_loop: int = None):
         if game_loop is not None:
             result = await self._execute(observation=sc_pb.RequestObservation(game_loop=game_loop))
         else:
@@ -182,7 +188,7 @@ class Client(Protocol):
     async def actions(self, actions, return_successes=False):
         if not actions:
             return None
-        elif not isinstance(actions, list):
+        if not isinstance(actions, list):
             actions = [actions]
 
         # On realtime=True, might get an error here: sc2.protocol.ProtocolError: ['Not in a game']
@@ -190,12 +196,11 @@ class Client(Protocol):
             res = await self._execute(
                 action=sc_pb.RequestAction(actions=(sc_pb.Action(action_raw=a) for a in combine_actions(actions)))
             )
-        except ProtocolError as e:
+        except ProtocolError:
             return []
         if return_successes:
             return [ActionResult(r) for r in res.action.result]
-        else:
-            return [ActionResult(r) for r in res.action.result if ActionResult(r) != ActionResult.Success]
+        return [ActionResult(r) for r in res.action.result if ActionResult(r) != ActionResult.Success]
 
     async def query_pathing(self, start: Union[Unit, Point2, Point3],
                             end: Union[Point2, Point3]) -> Optional[Union[int, float]]:
@@ -216,7 +221,7 @@ class Client(Protocol):
             return None
         return distance
 
-    async def query_pathings(self, zipped_list: List[List[Union[Unit, Point2, Point3]]]) -> List[Union[float, int]]:
+    async def query_pathings(self, zipped_list: List[List[Union[Unit, Point2, Point3]]]) -> List[float]:
         """Usage: await self.query_pathings([[unit1, target2], [unit2, target2]])
         -> returns [distance1, distance2]
         Caution: returns 0 when path not found
@@ -436,8 +441,6 @@ class Client(Protocol):
         """Moves camera to the target position using the spatial aciton interface
 
         :param position:"""
-        from s2clientprotocol import spatial_pb2 as spatial_pb
-
         assert isinstance(position, (Point2, Point3))
         action = sc_pb.Action(
             action_render=spatial_pb.ActionSpatial(
@@ -588,22 +591,27 @@ class Client(Protocol):
             if debug_hash != self._debug_hash_tuple_last_iteration:
                 # Something has changed, either more or less is to be drawn, or a position of a drawing changed (e.g. when drawing on a moving unit)
                 self._debug_hash_tuple_last_iteration = debug_hash
-                await self._execute(
-                    debug=sc_pb.RequestDebug(
-                        debug=[
-                            debug_pb.DebugCommand(
-                                draw=debug_pb.DebugDraw(
-                                    text=[text.to_proto() for text in self._debug_texts] if self._debug_texts else None,
-                                    lines=[line.to_proto()
-                                           for line in self._debug_lines] if self._debug_lines else None,
-                                    boxes=[box.to_proto() for box in self._debug_boxes] if self._debug_boxes else None,
-                                    spheres=[sphere.to_proto()
-                                             for sphere in self._debug_spheres] if self._debug_spheres else None,
+                try:
+                    await self._execute(
+                        debug=sc_pb.RequestDebug(
+                            debug=[
+                                debug_pb.DebugCommand(
+                                    draw=debug_pb.DebugDraw(
+                                        text=[text.to_proto()
+                                              for text in self._debug_texts] if self._debug_texts else None,
+                                        lines=[line.to_proto()
+                                               for line in self._debug_lines] if self._debug_lines else None,
+                                        boxes=[box.to_proto()
+                                               for box in self._debug_boxes] if self._debug_boxes else None,
+                                        spheres=[sphere.to_proto()
+                                                 for sphere in self._debug_spheres] if self._debug_spheres else None,
+                                    )
                                 )
-                            )
-                        ]
+                            ]
+                        )
                     )
-                )
+                except ProtocolError:
+                    return
             self._debug_draw_last_frame = True
             self._debug_texts.clear()
             self._debug_lines.clear()
@@ -732,19 +740,18 @@ class DrawItem:
         if color is None:
             return debug_pb.Color(r=255, g=255, b=255)
         # Need to check if not of type Point3 because Point3 inherits from tuple
-        elif isinstance(color, (tuple, list)) and not isinstance(color, Point3) and len(color) == 3:
+        if isinstance(color, (tuple, list)) and not isinstance(color, Point3) and len(color) == 3:
             return debug_pb.Color(r=color[0], g=color[1], b=color[2])
         # In case color is of type Point3
-        else:
-            r = getattr(color, "r", getattr(color, "x", 255))
-            g = getattr(color, "g", getattr(color, "y", 255))
-            b = getattr(color, "b", getattr(color, "z", 255))
-            if max(r, g, b) <= 1:
-                r *= 255
-                g *= 255
-                b *= 255
+        r = getattr(color, "r", getattr(color, "x", 255))
+        g = getattr(color, "g", getattr(color, "y", 255))
+        b = getattr(color, "b", getattr(color, "z", 255))
+        if max(r, g, b) <= 1:
+            r *= 255
+            g *= 255
+            b *= 255
 
-            return debug_pb.Color(r=int(r), g=int(g), b=int(b))
+        return debug_pb.Color(r=int(r), g=int(g), b=int(b))
 
 
 class DrawItemScreenText(DrawItem):
